package me.km.utils;

import java.lang.reflect.Field;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.init.Enchantments;
import net.minecraft.init.SoundEvents;
import net.minecraft.util.FoodStats;
import net.minecraft.util.SoundEvent;

public class ReflectionUtils 
{    
    // -----------------------------------------------------------------------------------
    // helper stuff
    // -----------------------------------------------------------------------------------
    
    private static <T> void setInt(Class<T> c, T t, String field, int i)
    {
        try
        {
            Field f = c.getDeclaredField(field);
            {
                try
                {
                    f.setAccessible(true);
                    f.setInt(t, i);
                }
                catch(SecurityException | IllegalArgumentException | IllegalAccessException ex)
                {
                    System.out.println(field + " - " + ex);
                }
            }
        }
        catch(NoSuchFieldException | SecurityException ex)
        {
            System.out.println(field + " - " + ex);
        }
    }
    
    private static <T> int getInt(Class<T> c, T t, String field, int error)
    {
        try
        {
            Field f = c.getDeclaredField(field);
            {
                try
                {
                    f.setAccessible(true);
                    return f.getInt(t);
                }
                catch(SecurityException | IllegalArgumentException | IllegalAccessException ex)
                {
                    System.out.println(field + " - " + ex);
                }
            }
        }
        catch(NoSuchFieldException | SecurityException ex)
        {
            System.out.println(field + " - " + ex);
        }
        return error;
    }
    
    private static <T> void setFloat(Class<T> c, T t, String field, float fl)
    {
        try
        {
            Field f = c.getDeclaredField(field);
            {
                try
                {
                    f.setAccessible(true);
                    f.setFloat(t, fl);
                }
                catch(SecurityException | IllegalArgumentException | IllegalAccessException ex)
                {
                    System.out.println(field + " - " + ex);
                }
            }
        }
        catch(NoSuchFieldException | SecurityException ex)
        {
            System.out.println(field + " - " + ex);
        }
    }
    
    private static <T> float getFloat(Class<T> c, T t, String field, float error)
    {
        try
        {
            Field f = c.getDeclaredField(field);
            {
                try
                {
                    f.setAccessible(true);
                    return f.getFloat(t);
                }
                catch(SecurityException | IllegalArgumentException | IllegalAccessException ex)
                {
                    System.out.println(field + " - " + ex);
                }
            }
        }
        catch(NoSuchFieldException | SecurityException ex)
        {
            System.out.println(field + " - " + ex);
        }
        return error;
    }
    
    private static <T> T getField(Class<T> cast, Class c, String field)
    {
        try
        {
            return (T) c.getField(field).get(null);
        }
        catch(NoSuchFieldException | SecurityException | IllegalAccessException | IllegalArgumentException ex)
        {
            return null;
        }
    }
    
    // -----------------------------------------------------------------------------------
    // Villager stuff
    // -----------------------------------------------------------------------------------
    
    public static void setCareerLevel(EntityVillager v, int level)
    {
        setInt(EntityVillager.class, v, "careerLevel", level);
    }
    
    // -----------------------------------------------------------------------------------
    // FoodStats
    // -----------------------------------------------------------------------------------
    
    public static void setExhaustion(FoodStats stats, float f)
    {
        setFloat(FoodStats.class, stats, "foodExhaustionLevel", f);
    }
    
    public static float getExhaustion(FoodStats stats)
    {
        return getFloat(FoodStats.class, stats, "foodExhaustionLevel", 0);
    }
    
    public static void setSaturation(FoodStats stats, float f)
    {
        setFloat(FoodStats.class, stats, "foodSaturationLevel", f);
    }
    
    public static float getSaturation(FoodStats stats)
    {
        return getFloat(FoodStats.class, stats, "foodSaturationLevel", 0);
    }
    
    // -----------------------------------------------------------------------------------
    // field list gets
    // -----------------------------------------------------------------------------------
    
    public static Enchantment getEnchantment(String name)
    {
        return getField(Enchantment.class, Enchantments.class, name);
    }
    
    public static SoundEvent getSoundEvent(String name)
    {
        return getField(SoundEvent.class, SoundEvents.class, name);
    }
}
